/**
  ******************************************************************************
  * @file    keypad.h
  * @author  D. Erickson
  * @version V0.0.1
  * @date    03/27/2011
  * @brief   Contains all keypad definitions: timing constants, command look-up-tables
  * 		 Etc.
  *
  * @history 03/19/11 dje	Keypad mapping OK
  * 		 03/27/11 dje	Sony IR timing
  * 		 04/10/11 dje   keymaps moved to keypad.c
  ******************************************************************************
  */


#ifndef __KEYMAP_H
#define __KEYMAP_H

//#include "stm32f10x.h"

/* IR pulse widths for 12 bit Sony format in uS*/
/* Start pulse is 2400us nom, 2000  min, 2800 max */
#define IR_ST_MIN	2000
#define IR_ST_MAX	2800
/* 0 pulse is 600us nom, 400  min, 800 max */
#define IR_LO_MIN	400
#define IR_LO_MAX   800
/* 1 pulse is 1200us nom, 1000  min, 1400 max */
#define IR_HI_MIN   1000
#define IR_HI_MAX   1400
/* IR timeout in milliseconds. */
#define IR_TIMEOUT 100

#define KEY_DEBOUNCE 	50		/* Keypad Debounce time in mS */
#define KEY_DELAY 		300		/* Keypad Delay before repeat time in mS */
#define KEY_REPEAT 		200		/* Keypad Repeat time time in mS */


/* Encoder action codes */
#define NADA 0
#define INC 1
#define DEC 2
#define ILL 3
/* Set to the appropriate encoder used */
//#define ENC4PPCLICK
#define ENC1PPCLICK

#endif /* __KEYMAP_H */
