/**
  ******************************************************************************
  * @file    lcd_ks0108.h
  * @author  D. Erickson
  * @version V0.1
  * @date    02/2/2011
  * @brief   Header file for LCD 128 x 64 with KS0108 on Discovery Board
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __LCD_KS0108_H
#define __LCD_KS0108_H

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x.h"


extern void Delay(__IO uint32_t nCount);

/** @addtogroup STM32_LCD_KS108
  * @{
  *  LCD control pins and ports
  */

#define LCD_DB_PORT		GPIOC  /* LCD Data Bus */
#define LCD_CTL_PORT	GPIOC  /* LCD Control pins */
#define LCD_DB_PINS     GPIO_Pin_7|GPIO_Pin_6|GPIO_Pin_5|GPIO_Pin_4|GPIO_Pin_3|GPIO_Pin_2|GPIO_Pin_1|GPIO_Pin_0
#define LCD_DB_OUT		0x33333333  /* Bus direction control for OUT */
#define LCD_DB_IN		0x44444444  /* Bus direction control for OUT */
#define LCD_DB_CLK      RCC_APB2Periph_GPIOC
#define LCD_E_PIN       GPIO_Pin_13
#define LCD_E_CLK       RCC_APB2Periph_GPIOC
#define LCD_RS_PIN      GPIO_Pin_9
#define LCD_RS_CLK      RCC_APB2Periph_GPIOC
#define LCD_RW_PIN      GPIO_Pin_10
#define LCD_RW_CLK      RCC_APB2Periph_GPIOC
#define LCD_CS1_PIN     GPIO_Pin_11
#define LCD_CS1_CLK     RCC_APB2Periph_GPIOC
#define LCD_CS2_PIN     GPIO_Pin_12
#define LCD_CS2_CLK     RCC_APB2Periph_GPIOC
/* KS0108 commands */
#define LCD_DISP_ON		0x3F
#define LCD_DISP_OFF	0x3E
#define LCD_START_0		0xC0
#define LCD_XADDR		0x40
#define LCD_YADDR		0xB8
/* LCD dimensions */
#define LCD_XSIZ		128
#define LCD_YSIZ		64
#define LCD_XMAX		127
#define LCD_YMAX		63
/* Shortcuts for character rows of the display. */
#define ROW_0			(0*8)
#define ROW_1			(1*8)
#define ROW_2			(2*8)
#define ROW_3			(3*8)
#define ROW_4			(4*8)
#define ROW_5			(5*8)
#define ROW_6			(6*8)
#define ROW_7			(7*8)

/** @defgroup STM32_Discovery_LOW_LEVEL__Exported_Functions
  * @{
  */
void 	LCD_GPIOConfiguration(void);
void 	lcd_Init(void);
void 	lcd_cmd(uint8_t data);
void 	lcd_data(uint8_t data, uint8_t chip);
void 	lcd_w(uint8_t x, uint8_t y, uint8_t data);
uint8_t lcd_r(uint8_t x, uint8_t y);
void 	lcd_clear_line(int32_t line, uint8_t value);
void 	lcd_clear(uint8_t value);
void 	lcd_char(uint8_t x, uint8_t y, char ch);
void 	lcd_string(uint8_t x, uint8_t y, char *str);
uint8_t lcd_rdata(uint8_t chip);
void 	lcd_putpix( uint8_t x, uint8_t y, uint8_t data);
void 	lcd_line(int32_t startx, int32_t starty, int32_t endx, int32_t endy);
void 	lcdCircle( int32_t xc, int32_t yc, int32_t xrad, int32_t yrad);
void 	lcd_bar(uint16_t xs, uint16_t ys, int16_t len);
void 	lcd_rect(uint16_t xs, uint16_t ys, uint16_t xsiz, uint16_t ysiz);
void 	pulse_E(void);
void 	lcdTest(void);

#endif /* __LCD_KS0108_H */

