/* The Dispatch function, display, and setInput() routines live here.
 * For each key code, the code that executes that command. One big case statement.
 * Also the text label defines for the zones and inputs are here.
 */

#include <stdio.h>
#include "main.h"
#include "lcd_ks0108.h"
#include "i2c.h"


extern inKeyTypeDef inKey[];

/* Text labels for the zones and inputs (sources) */
const char *chanLabel[]= {
    "LIV RM",
    "DEN   ",
    "BR1   ",
    "BR2 M ",
    "BASE  ",
    "DECK  ",
    "KIT   ",
    "BATH  "
    };

const char *sourceLabel[]= {
    "CD  ",
    "PHON",
    "TUN ",
    "TV  ",
    "PC  ",
    "DVD ",
    "MP3 ",
    "AUX1"
    };


/* Dispatch executes the commands. S
 * Begins by setting the zone for that key channel keyChan */
void dispatch(uint8_t chan, uint8_t cmd)
{
	uint8_t key;
	uint8_t src;				/* Input source variable */
	uint8_t zone;

	zone = inKey[chan].zone;	/* Set the zone */
//	dispZone = zone;
	key = cmd;

/* Big case statement */
switch(key)
    {
	/* Source selections */
	case '1':	/* source = 0 */
	{
		setSource(zone, 0);
	}
	break;
    case '2':	/* source = 1 */
	{
		setSource(zone, 1);
	}
	break;
    case '3':	/* source = 2 */
	{
		setSource(zone, 2);
	}
	break;
    case '4':	/* source = 3 */
	{
		setSource(zone, 3);
	}
	break;
    case '5':	/* source = 4 */
	{
		setSource(zone, 4);
	}
	break;
    case '6':	/* source = 5 */
	{
		setSource(zone, 5);
	}
	break;
    case '7':	/* source = 6 */
	{
		setSource(zone, 6);
	}
	break;
    case '8':	/* source = 7 */
	{
		setSource(zone, 7);
	}
	break;

	/* Zone selections */
    case '!':	/* zone 0 */
    {
    	setZone(chan, 0);
    }
	break;
    case '@':	/* zone 1 */
	{
		setZone(chan, 1);
	}
	break;
    case '#':	/* zone 2 */
	{
		setZone(chan, 2);
	}
	break;
    case '$':	/* zone 3 */
	{
		setZone(chan, 3);
	}
	break;
    case '%':	/* zone 4 */
	{
		setZone(chan, 4);
	}
	break;
    case '^':	/* zone 5 */
	{
		setZone(chan, 5);
	}
	break;
    case '&':	/* zone 6 */
	{
		setZone(chan, 6);
	}
	break;
    case '*':	/* zone 7 */
	{
		setZone(chan, 7);
	}
	break;

    case 'n':	/* Select next source (increment)*/
	{
		src = preamp[zone].source + 1;
		if (src > SOURCE_MAX)
		{
			src = 0;
		}
		setSource(zone, src);
	}
	break;

    case 'p':	/* previous source (decrement) */
	{
		src = preamp[zone].source - 1;
		if (src > SOURCE_MAX)
		{
			src = SOURCE_MAX;
		}
	 	setSource(zone, src);
	}
	break;

    case 'N':	    /* Next zone */
	{
		inKey[chan].zone ++;
		if(inKey[chan].zone > ZONE_MAX)
		{
			inKey[chan].zone = 0;
		}
	}
	break;

    case 'P':	    /* Previous zone */
    {
    	inKey[chan].zone--;
    	if(inKey[chan].zone > ZONE_MAX)		/* A hack: this means it wrapped around */
    	{
    		inKey[chan].zone = ZONE_MAX;
    	}
    }
	break;

    case 'V':	    /* Volume up */
	{
		preamp[zone].vol += VOL_STEP;
		if (preamp[zone].vol > VOL_MAX) preamp[zone].vol = VOL_MAX;
		muteOff(zone);
		preamp_set(zone);						/* Set the preamp */
	}
	break;

    case 'v':	    /* Volume down */
	{
		preamp[zone].vol -= VOL_STEP;
		if (preamp[zone].vol < VOL_MIN) preamp[zone].vol = VOL_MIN;
		muteOff(zone);
		preamp_set(zone);						/* Set the preamp */
	}
	break;

    case 'r':	    /* balance right */
	{
		preamp[zone].bal += BAL_STEP;
		if (preamp[zone].bal > BAL_MAX) preamp[zone].bal = BAL_MAX;
		muteOff(zone);
		preamp_set(zone);						/* Set the preamp */
	}
	break;

    case 'l':	    /* balance left */
	{
		preamp[zone].bal -= BAL_STEP;
		if (preamp[zone].bal < BAL_MIN) preamp[zone].bal = BAL_MIN;
		muteOff(zone);
		preamp_set(zone);						/* Set the preamp */
	}
	break;

    case 'T':	    /* treble up  */
	{
		preamp[zone].treb += TONE_STEP;
		if (preamp[zone].treb > TONE_MAX) preamp[zone].treb = TONE_MAX;
		muteOff(zone);
		preamp_set(zone);						/* Set the preamp */
	}
	break;

    case 't':	    /* treble down  */
	{
		preamp[zone].treb -= TONE_STEP;
		if (preamp[zone].treb < TONE_MIN) preamp[zone].treb = TONE_MIN;
		muteOff(zone);
		preamp_set(zone);						/* Set the preamp */
	}
	break;

    case 'B':	    /* bass up  */
	{
		preamp[zone].bass += TONE_STEP;
		if (preamp[zone].bass > TONE_MAX) preamp[zone].bass = TONE_MAX;
		muteOff(zone);
		preamp_set(zone);						/* Set the preamp */
	}
	break;

    case 'b':	    /* bass down */
	{
		preamp[zone].bass -= TONE_STEP;
		if (preamp[zone].bass < TONE_MIN)  preamp[zone].bass = TONE_MIN;
		muteOff(zone);
		preamp_set(zone);						/* Set the preamp */
	}
	break;

    case 'M':	    /* midrange up */
 	{
 		preamp[zone].mid += TONE_STEP;
 		if (preamp[zone].mid > TONE_MAX) preamp[zone].mid = TONE_MAX;
 		muteOff(zone);
 		preamp_set(zone);						/* Set the preamp */
 	}
 	break;

    case 'm':	    /* midrange down */
 	{
 		preamp[zone].mid -= TONE_STEP;
 		if (preamp[zone].mid < TONE_MIN)  preamp[zone].mid = TONE_MIN;
 		muteOff(zone);
 		preamp_set(zone);						/* Set the preamp */
 	}
 	break;

    case 'h':	    /* Help displays to the UART*/
	{
		putString("1-8: Select input  !-*: Zone\n\r");
		putString("V v: Vol  r l: Bal  B b: Bass M m: Mid T t: Treb\n\r");
		putString("h: Help(this screen) d: display struct  x: mute\n\r");
	}
	break;

    case 'd':	    /* Print channel struct */
	{
		dispStruct(zone);
	}
	break;

    case 'q':	    /* Toggle Mute */
	{
		if (preamp[zone].mute == 0) preamp[zone].mute = 1;
		else preamp[zone].mute = 0;
		preamp_set(zone);
	}
	break;

    case 'Q':	    /* Mute all non-fixed-gain channels */
 	{
 		for (zone = 0; zone < (ZONE_MAX + 1); zone++)
 		{
 			if(preamp[zone].fixedGain == 0)
 			{
 				preamp[zone].mute = 1;
 				preamp_set(zone);
 			}
 		}
 	}
	break;

    default:
	break;

    }
}

void muteOff(uint8_t zone)
{
	preamp[zone].mute = 0;
}

/* Set the input channel to 'in' */
void setSource(uint8_t zone, uint8_t src)
{
	uint8_t old_source;

	old_source = preamp[zone].source;		/* Get the previous setting */
	preamp[zone].source = src ;				/* Set the new */
	xpt_set(zone, old_source, src);			/* Set the crosspoint */
 	muteOff(zone);							/* Turn off mute, set preamp */
 	preamp_set(zone);
}

/* Set current zone */
void setZone(uint8_t chan, uint8_t zone)
{
	inKey[chan].zone = zone;
}


/* Display the channel struct on the LCD */
void test_update(uint8_t chan)
{
	char buf1[32];
	int8_t v, s, b, m, t, bal;

	s = preamp[chan].source;
	v = preamp[chan].vol;
	b = preamp[chan].bass;
	t = preamp[chan].treb;
	m = preamp[chan].mid;
	bal = preamp[chan].bal;

//	lcd_clear(0);

	lcd_string(0, ROW_0, "  Multi-Zone Stereo");

	sprintf(buf1,"Zone:%d Src:%d", chan, s);
	lcd_string(0, ROW_3, buf1);
	sprintf(buf1,"Vol:%d Bal:%d", v, bal);
	lcd_string(0, ROW_5, buf1);
	sprintf(buf1,"Bass:%d Mid:%d Treb:%d", b, m, t);
	lcd_string(0, ROW_7, buf1);
}

void dispStruct(uint8_t chan)
{
	char buf1[64];
	int8_t v, s, b, m, t, bal, mu;

	s = preamp[chan].source;
	v = preamp[chan].vol;
	b = preamp[chan].bass;
	t = preamp[chan].treb;
	m = preamp[chan].mid;
	bal = preamp[chan].bal;
	mu = preamp[chan].mute;

	sprintf(buf1,"Z=%d:S=%d:S=%d:Bal=%d:B=%d:M=%d:M=%d:Mu=%d\n\r", chan,s,v,bal,b,m,t,mu);
	putString(buf1);
}

/* Init all the preamp structures to default settings.
 * If there was NV RAM, these could be updated from it.
 * 7/30 Fixed gain channels now default to full volume.
 *
 * */
void initStruct(void)
{
	uint8_t chan;

	preamp[1].fixedGain = 1;			/* These are the fixed gain channels */
	preamp[2].fixedGain = 1;
	preamp[3].fixedGain = 1;

	for(chan=0; chan < ZONE_MAX+1; chan++)
	{
		preamp[chan].source = 0;
		if (preamp[chan].fixedGain == 1) /* Set fixed channels to VOL_MAX */
			preamp[chan].vol = VOL_MAX;
		else
			preamp[chan].vol = VOL_MIN;
		preamp[chan].bass = 0;
		preamp[chan].treb = 0;
		preamp[chan].mid = 0 ;
		preamp[chan].bal = 0;
		preamp[chan].mute = 0 ;
		preamp[chan].loudness = 0;
	}
}


/* Graphics update. Here is the display format by row:
 * ------------------------------------------
 * 0	ZONE=    INPUT=
 * 1	MUTE
 * 2	VOL  --------------
 * 3	BAL       |--
 * 4	BASS      |--
 * 5	MID       |--
 * 6	TREB      |--
 * 7    (spare)
 * ---------------------------------
 * Clearing the area of the display prior to displaying a bar is important.
 * Currently this is done by lcd_string() which clears the line to the right.
 * To display graphics only, you could clear the row by displaying a null string "".
 */
void update(uint8_t chan)
{
	char buf1[32];
	int8_t v, s, b, m, t, bal;
//	int8_t mu;
	int16_t vbar, balbar, bbar, mbar, tbar;		/* Bar lengths */
	int16_t row = 0;

	int16_t BARSTART = 48;
	int16_t BARMID = (BARSTART + LCD_XMAX) / 2 ;	/* Center position for the signed bars */

	/* Get all the parameters from the channel structure */
	s = preamp[chan].source;
	v = preamp[chan].vol;
	b = preamp[chan].bass;
	t = preamp[chan].treb;
	m = preamp[chan].mid;
	bal = preamp[chan].bal;
//	mu = preamp[chan].mute;

	/* Scale the bar graph lengths for the display. These should be #defines */
	vbar   = ((127 -BARSTART - 1) * (-VOL_MIN + v)) / (-VOL_MIN);			/* 150% */
	balbar = (120 * bal) / 100;
	bbar   = (250 * b)   / 100;
	mbar   = (250 * m)   / 100;
	tbar   = (250 * t)   / 100;

	//	lcd_clear(0);		/* No need to clear first since we're writing the whole display */

	sprintf(buf1,"Zone=%s  Src=%s", chanLabel[chan], sourceLabel[s]);
	lcd_string(0, row , buf1);

	if (preamp[chan].mute == 1)
	{
		sprintf(buf1,"Mute");					/* Display Mute */
	}
	else
	{
		sprintf(buf1," ");						/* if not mute, display empty line */
	}
	lcd_string(0, row += 8, buf1);

	sprintf(buf1,"Vol  %d", v);					/* Display each parameter, then a bar */
	lcd_string(0, row += 8, buf1);
	lcd_bar (BARSTART, row, vbar);

	sprintf(buf1,"Bal  %d", bal);
	lcd_string(0, row += 8, buf1);
	lcd_bar (BARMID, row, balbar);

	sprintf(buf1,"Bass %d", b);
	lcd_string(0, row += 8, buf1);
	lcd_bar (BARMID, row, bbar);

	sprintf(buf1,"Mid  %d ", m);
	lcd_string(0, row += 8, buf1);
	lcd_bar (BARMID, row, mbar);

	sprintf(buf1,"Treb %d", t);
	lcd_string(0, row += 8, buf1);
	lcd_bar (BARMID, row, tbar);

	sprintf(buf1," ");							/* Empty line */
	lcd_string(0, row += 8, buf1);

	/* put a box around the bars */
	lcd_rect(BARSTART - 1, 15, LCD_XMAX - BARSTART - 1, 56 - 15);
}


